import numpy as np
import pandas as pd
from sklearn.compose import ColumnTransformer

class columnTransformer(ColumnTransformer):
    def _hstack(self, Xs):
        cols = [X.columns.tolist() for X in Xs]
        cols = np.hstack(cols)
        return pd.DataFrame(super()._hstack(Xs), columns = cols)




import pandas as pd
from sklearn.impute import SimpleImputer


class simpleImputer(SimpleImputer):
    def transform(self, X):
        data = super().transform(X)
        return pd.DataFrame(data, columns = X.columns, index = X.index)



import numpy as np
import pandas as pd
from sklearn.base import BaseEstimator, TransformerMixin
from sklearn.preprocessing import PolynomialFeatures
from sklearn.preprocessing import StandardScaler, RobustScaler

class polynomialFeatures(PolynomialFeatures):
    def transform(self, X):
        data = super().transform(X)
        cols = self.get_feature_names(input_features = X.columns.tolist())
        return pd.DataFrame(data, columns = cols, index = X.index)


class removeDuplicate(BaseEstimator, TransformerMixin):
    def __init__(self):
        super().__init__()
        self.unique_indices_
    def fit(self, X, y = None):
        _, self.unique_indices_ = np.unique(X, axis=1, return_index=True)
        return self
    def transform(self, X, y = None):
        return X.iloc[:, self.unique_indices_]

class standardScaler(StandardScaler):
    def transform(self, X):
        data = super().transform(X)
        return pd.DataFrame(data, columns = X.columns, index = X.index)
class robustScaler(RobustScaler):
    def transform(self, X):
        data = super().transform(X)
        return pd.DataFrame(data, columns = X.columns, index = X.index)



import numpy as np
import pandas as pd
from sklearn.base import BaseEstimator, TransformerMixin
from sklearn.preprocessing import KBinsDiscretizer

class Binning(BaseEstimator, TransformerMixin):
    def __init__(self, features_discretize = None):
        super().__init__()
        self.features_to_discretize = features_discretize
        self.disc = None
    def fit(self, X, y = None):
        X = X.copy()
        if self.features_to_discretize is None:
            self.features_to_discretize = X.columns.tolist()
        if len(self.features_to_discretize) > 0:
            self.binns = []
            for col in self.features_to_discretize:
                hist, _ = np.histogram(X[col], bins = 'sturges')
                self.binns.append(len(hist))

            self.len_columns = len(self.features_to_discretize)
            self.disc = KBinsDiscretizer(n_bins = self.binns,encode = 'ordinal',
                strategy = 'kmeans')
            self.disc.fit(X[self.features_to_discretize].values.reshape(-1, self.len_columns))
        return self
    def transform(self, X, y = None):
        X = X.copy()
        if self.disc is not None:
            X[self.features_to_discretize] = self.disc.transform(X[self.features_to_discretize].values.reshape(-1, self.len_columns))
            data = X[self.features_to_discretize]
            data.columns = [f'bin_{col}' for col in data.columns]
            return data
        return X



import numpy as np
import pandas as pd
from sklearn.feature_selection import SequentialFeatureSelector

class sequentialFeatureSelector(SequentialFeatureSelector):
    def transform(self, x):
        mask = self.get_support()
        columns = x.loc[:, mask].columns.tolist()
        data = super().transform(x)
        return pd.DataFrame(data, columns = columns, index = x.index)




import numpy as np
import pandas as pd
from sklearn.kernel_approximation import PolynomialCountSketch

class polynomialCountSketch(PolynomialCountSketch):
    feature_name = 'pcsketch'
    def transform(self, X):
        data = super().transform(X)
        cols = [f'{self.feature_name}_{i}' for i in range(data.shape[1])]
        return pd.DataFrame(data, columns = cols, index = X.index)
